<?php

namespace App\Http\Controllers\User;

use App\Constants\Status;
use App\Events\AcceptJoinRequest;
use App\Events\CancelWatchParty;
use App\Events\ConversationMessage;
use App\Events\PlayerSetting;
use App\Events\RejectJoinRequest;
use App\Events\ReloadWatchParty;
use App\Events\SendJoinWatchParty;
use App\Http\Controllers\Controller;
use App\Models\Conversation;
use App\Models\PartyMember;
use App\Models\WatchParty;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Validator;

class WatchPartyController extends Controller {
    public function create(Request $request) {
        $validator = Validator::make($request->all(), [
            'party_code' => 'required|string|max:8',
        ]);

        if ($validator->fails()) {
            return response()->json(['error' => 'Party code not found']);
        }

        $user = auth()->user();
        if (now() > $user->exp) {
            return response()->json(['error' => 'Please subscribe a plan for watch party']);
        }

        $running = WatchParty::active()->where('user_id', $user->id)->exists();
        if ($running) {
            return response()->json(['error' => 'You have created already a watch party room']);
        }

        $existRoom = WatchParty::inactive()->where('party_code', $request->party_code)->where('user_id', $user->id)->exists();
        if ($existRoom) {
            return response()->json(['error' => 'This party has been canceled']);
        }

        $watchParty             = new WatchParty();
        $watchParty->user_id    = $user->id;
        $watchParty->item_id    = $request->item_id;
        $watchParty->episode_id = @$request->episode_id ?? 0;
        $watchParty->party_code = $request->party_code;
        $watchParty->save();

        return response()->json([
            'redirect_url' => route('user.watch.party.room', $watchParty->party_code),
        ]);
    }

    public function room($code, $guestId = 0) {

        $user      = auth()->user();
        $partyRoom = WatchParty::active();
        if (!$guestId) {
            $partyRoom->where('user_id', $user->id);
        }
        $partyRoom = $partyRoom->where('party_code', $code)->with(['item', 'episode', 'user'])->first();

        if (!$partyRoom) {
            $notify[] = ['error', 'Invalid watch party room request'];
            return to_route('user.home')->withNotify($notify);
        }

        if ($partyRoom->user_id != $user->id) {
            $partyMember = PartyMember::accepted()->where('user_id', $user->id)->where('watch_party_id', $partyRoom->id)->first();
            if (!$partyMember) {
                $notify[] = ['error', 'Access denied permission to watch party'];
                return to_route('user.home')->withNotify($notify);
            }
        }

        if ($partyRoom->episode) {
            $video = $partyRoom->episode->video;
            $item  = $partyRoom->episode->item;
        } else {
            $video = $partyRoom->item->video;
            $item  = $partyRoom->item;
        }
        if ($item->version == Status::PAID_VERSION && now() > $user->exp) {
            $notify[] = ['error', 'Please subscribe a plan for watch party'];
            return to_route('user.home')->withNotify($notify);
        }
        $subtitles = $video->subtitles;
        $videos    = $this->videoList($video);

        $pageTitle     = 'Watch Party Room - ' . $partyRoom->party_code;
        $conversations = Conversation::where('watch_party_id', $partyRoom->id)->with('user')->latest()->limit(10)->get();
        $partyMembers  = PartyMember::accepted()->where('watch_party_id', $partyRoom->id)->with('user')->get();
        return view($this->activeTemplate . 'user.watch_party.room', compact('partyRoom', 'pageTitle', 'videos', 'subtitles', 'item', 'conversations', 'partyMembers'));
    }

    private function videoList($video) {
        $videoFile = [];
        if ($video->three_sixty_video) {
            $videoFile[] = [
                'content' => getVideoFile($video, 'three_sixty'),
                'size'    => 360,
            ];
        }
        if ($video->four_eighty_video) {
            $videoFile[] = [
                'content' => getVideoFile($video, 'four_eighty'),
                'size'    => 480,
            ];
        }
        if ($video->seven_twenty_video) {
            $videoFile[] = [
                'content' => getVideoFile($video, 'seven_twenty'),
                'size'    => 720,
            ];
        }
        if ($video->thousand_eighty_video) {
            $videoFile[] = [
                'content' => getVideoFile($video, 'thousand_eighty'),
                'size'    => 1080,
            ];
        }
        return json_decode(json_encode($videoFile, true));
    }
    public function joinRequest(Request $request) {
        $validator = Validator::make($request->all(), [
            'party_code' => 'required|string',
        ]);

        if ($validator->fails()) {
            return response()->json([
                'error' => $validator->errors()->all(),
            ]);
        }

        $party = WatchParty::active()->where('party_code', $request->party_code)->with(['item', 'episode'])->first();
        if (!$party) {
            return response()->json([
                'redirect' => false,
                'error', 'Invalid join request',
            ]);
        }

        if ($party->episode) {
            $item = $party->episode;
        } else {
            $item = $party->item;
        }

        $user = auth()->user();
        if ($item->version == Status::PAID_VERSION && now() > $user->exp) {
            return response()->json([
                'redirect' => true,
                'error'    => 'You have to subscribe any plan',
            ]);
        }

        $alreadyJoined = PartyMember::accepted()->where('user_id', $user->id)->where('watch_party_id', $party->id)->first();

        if ($alreadyJoined) {
            return response()->json([
                'redirect_url' => route('user.watch.party.room', [$party->party_code, $alreadyJoined->user_id]),
            ]);
        }

        $member                 = new PartyMember();
        $member->watch_party_id = $party->id;
        $member->user_id        = $user->id;
        $member->save();

        $this->initializePusher();
        event(new SendJoinWatchParty($party, $member->id));

        return response()->json([
            'redirect' => false,
            'host_id'  => $party->user_id,
            'success'  => 'Joining request send successfully',
        ]);
    }

    private function initializePusher() {
        $general = gs();
        Config::set('broadcasting.connections.pusher.driver', 'pusher');
        Config::set('broadcasting.connections.pusher.key', $general->pusher_config->app_key);
        Config::set('broadcasting.connections.pusher.secret', $general->pusher_config->app_secret_key);
        Config::set('broadcasting.connections.pusher.app_id', $general->pusher_config->app_id);
        Config::set('broadcasting.connections.pusher.option.cluster', $general->pusher_config->cluster);
    }

    public function requestAccept($id) {
        $partyMember = PartyMember::where('id', decrypt($id))->first();
        if (!$partyMember) {
            return response()->json(['error' => 'Member not found']);
        }
        $party = WatchParty::active()->where('user_id', auth()->id())->where('id', $partyMember->watch_party_id)->first();
        if (!$party) {
            return response()->json(['error' => 'Invalid Request']);
        }

        $member = $partyMember->user;
        if ($party->episode) {
            $item = $party->episode;
        } else {
            $item = $party->item;
        }

        if ($item->version === Status::PAID_VERSION && (!$member->plan || now() > $member->exp)) {
            return response()->json(['error' => 'This member has no subscription']);
        }

        $partyMember->status = Status::WATCH_PARTY_REQUEST_ACCEPTED;
        $partyMember->save();

        $this->initializePusher();
        event(new AcceptJoinRequest($party, $partyMember));

        return response()->json([
            'success' => 'Joining request has been accepted',
        ]);
    }

    public function requestReject($id) {
        $partyMember = PartyMember::where('id', decrypt($id))->first();
        if (!$partyMember) {
            return response()->json(['error' => 'Member not found']);
        }
        $party = WatchParty::active()->where('user_id', auth()->id())->where('id', $partyMember->watch_party_id)->first();
        if (!$party) {
            return response()->json(['error' => 'Invalid Request']);
        }

        $partyMember->status = Status::WATCH_PARTY_REQUEST_REJECTED;
        $partyMember->save();

        $this->initializePusher();
        event(new RejectJoinRequest(route('user.home'), $partyMember->user_id));

        return response()->json([
            'success' => 'Joining request has been rejected',
        ]);
    }

    public function sendMessage(Request $request) {
        $validator = Validator::make($request->all(), [
            'message'  => 'required|string',
            'party_id' => 'required|integer|exists:watch_parties,id',
        ]);

        if ($validator->fails()) {
            return response()->json([
                'error' => $validator->errors()->all(),
            ]);
        }

        $party = WatchParty::active()->with('partyMember')->where('id', $request->party_id)->first();
        if (!$party) {
            return response()->json(['error', 'Party room not found']);
        }

        $joinedUsersId = $party->partyMember->pluck('user_id')->unique()->toArray();
        $hostId        = $party->user_id;
        $allMemberId   = array_merge([$hostId], $joinedUsersId);

        $user = auth()->user();
        if (!in_array($user->id, $allMemberId)) {
            return response()->json(['error', 'Access denied for conversation']);
        }

        $conversation                 = new Conversation();
        $conversation->user_id        = $user->id;
        $conversation->watch_party_id = $party->id;
        $conversation->message        = $request->message;
        $conversation->save();

        $this->initializePusher();
        event(new ConversationMessage($conversation, $allMemberId));
        return response()->json([
            'success' => true,
        ]);
    }

    public function playerSetting(Request $request) {
        $validator = Validator::make($request->all(), [
            'party_id' => 'required|integer|exists:watch_parties,id',
            'status'   => 'required|string|in:play,pause',
        ]);

        if ($validator->fails()) {
            return response()->json([
                'error' => $validator->errors()->all(),
            ]);
        }

        $party = WatchParty::active()->with('partyMember')->where('id', $request->party_id)->first();
        if (!$party) {
            return response()->json(['error', 'Party room not found']);
        }

        $joinedUsersId = $party->partyMember->pluck('user_id')->unique()->toArray();
        $hostId        = $party->user_id;
        $allMemberId   = array_merge([$hostId], $joinedUsersId);

        $this->initializePusher();
        event(new PlayerSetting($allMemberId, $request->status));

        return response()->json(['success' => true]);
    }

    public function status($id) {
        $partyMember = PartyMember::accepted()->where('id', $id)->whereHas('watchParty', function ($query) {
            $query->where('user_id', auth()->id());
        })->first();

        if (!$partyMember) {
            return response()->json(['error' => 'Member not found']);
        }

        $partyMember->status = Status::WATCH_PARTY_REQUEST_REJECTED;
        $partyMember->save();

        $this->initializePusher();
        event(new RejectJoinRequest(route('user.home'), $partyMember->user_id));

        return response()->json([
            'success' => 'Member removed successfully',
            'id'      => $partyMember->id,
        ]);
    }

    public function cancel($id) {
        $party = WatchParty::where('user_id', auth()->id())->where('id', $id)->first();
        if (!$party) {
            return response()->json(['error' => 'Party not found']);
        }
        $party->status = Status::DISABLE;
        $party->save();

        $members = $party->partyMember;
        foreach ($members as $member) {
            $member->status = Status::WATCH_PARTY_REQUEST_REJECTED;
            $member->save();
        }

        $joinedUsersId = $party->partyMember->pluck('user_id')->unique()->toArray();
        $hostId        = $party->user_id;
        $allMemberId   = array_merge([$hostId], $joinedUsersId);

        $this->initializePusher();
        event(new CancelWatchParty(route('user.home'), $allMemberId));

        return response()->json(['success' => true]);
    }

    public function history() {
        $pageTitle = 'Watch Party History';
        $parties   = WatchParty::where('user_id', auth()->id())
            ->with('item:id,title', 'episode:id,title')
            ->with(['partyMember' => function ($query) {
                $query->select('user_id', 'watch_party_id')->distinct('user_id');
            }])
            ->latest()
            ->paginate(getPaginate());
        return view($this->activeTemplate . 'user.watch_party.history', compact('pageTitle', 'parties'));
    }

    public function disabled($id) {
        $party = WatchParty::where('user_id', auth()->id())->where('id', $id)->first();
        if (!$party) {
            $notify[] = ['error', 'Party not found'];
            return back()->withNotify($notify);
        }
        $party->status = Status::DISABLE;
        $party->save();

        $members = $party->partyMember;
        foreach ($members as $member) {
            $member->status = Status::WATCH_PARTY_REQUEST_REJECTED;
            $member->save();
        }

        $joinedUsersId = $party->partyMember->pluck('user_id')->unique()->toArray();
        $hostId        = $party->user_id;
        $allMemberId   = array_merge([$hostId], $joinedUsersId);

        $this->initializePusher();
        event(new CancelWatchParty(route('user.home'), $allMemberId));

        $notify[] = ['success', 'Party canceled successfully'];
        return back()->withNotify($notify);
    }

    public function reload(Request $request) {
        $validator = Validator::make($request->all(), [
            'party_id' => 'required|integer|exists:watch_parties,id',
        ]);

        if ($validator->fails()) {
            return response()->json([
                'error' => $validator->errors()->all(),
            ]);
        }
        $party = WatchParty::where('user_id', auth()->id())->where('id', $request->party_id)->first();
        if (!$party) {
            $notify[] = ['error', 'Party not found'];
            return back()->withNotify($notify);
        }
        $joinedUsersId = $party->partyMember->pluck('user_id')->unique()->toArray();
        $this->initializePusher();
        event(new ReloadWatchParty($joinedUsersId));

        return response()->json(['success' => true]);
    }
}
